from setuptools import setup

setup(
    name='foxmetrics',
    version='1.0.9',
    packages=['foxmetrics'],
    url='',
    license='',
    author='jizong',
    author_email='jizong.peng.ca@gmail.com',
    description='metrics for decoupled training',
    requires=["loguru", "termcolor"]
)
